; Output multi-ports

(provide 'omp)
(require 'lib)
(require 'ref)
(require 'types)
(require 'stk)

(define omp::mark (types:make-mark 'omp))
(define (omp? o)
   (and (pair? o) (eq? (car o) omp::mark)))
(types:register 'omp omp?)

(define (omp:new)
   (list omp::mark (stk:new) (stk:new)))

(define omp::current (omp:new))

(define (with-omp omp thunk)
   (let ((saved-omp omp::current))
   (dynamic-wind
      (lambda () (set! omp::current omp))
      thunk
      (lambda () (set! omp::current saved-omp)))))

(define (with-new-omp thunk)
    (let ((omp (omp:new)))
    (with-omp omp thunk)
    (omp:flush omp)))

(define (omp:flush o)
   (cond
      ((omp? o)
         (for-each omp:flush (reverse (stk->list (cadr o))))
	 (for-each omp:flush (stk->list (caddr o))))
      ((string? o)
         (display o))
      ((char? o)
         (display o))
      (else
         (ERROR "That's not a valid omp"))))

(define (omp:flush!)
   (omp:flush omp::current)
   (set! omp::current (omp:new)))

(define (omp:put! x)
   (stk:push! (cadr omp::current) x))

(define (omp:push! x)
   (stk:push! (caddr omp::current) x))

(define (omp:fork!)
   (let ((x (omp:new)))
      (omp:put! x)
      x))
