; references
;; All objects are meant to be read-only, but for explicit references
;;
;; We trust the program not to modify anything but this emulated type...


(provide 'ref)
(require 'lib)
(require 'types)

(define ref::mark (types:make-mark 'ref))

(define (ref:new . rest)
   "make a variable reference object,
that be initialized to point to object obj"
   (let ((obj (if (not (null? rest)) (car rest))))
   (cons ref::mark obj)))

(define (ref? o)
   (and (pair? o) (eq? (car o) ref::mark)))

(define (ref:get r)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (cdr r))

(define (ref:set! r v)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (set-cdr! r v))

(define (ref:map! r f)
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (let ((v (cdr r)))
     (set-cdr! r (f v))
     v))

(define (ref:map! r f)
   "replaces the referenced element by its image by a given function"
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (set-cdr! r (f (cdr r))))

(define (ref:postmap! r f)
   "replaces the referenced element by its image by a given function"
   "and returns the previous value"
   (ASSERT ref? r)
   ; make it atomic wrt GC
   (let ((v (cdr r)))
     (set-cdr! r (f v))
     v))

(define (ref:apply! r f . x)
   (ref:map! r (lambda (v) (apply f (cons v x)))))

(define (ref:postapply! r f . x)
   (ref:map! r (lambda (v) (apply f (cons v x)))))

(define (ref:++ r)
   (ref:postmap! r 1+))

;;;;; Counters

(define (new-counter)
  (let ((cnt (ref:new 0)))
  (lambda () (ref:++ cnt))))

(types:register 'ref ref?)
